﻿//EventDispatcher needs this
import mx.events.EventDispatcher;
//XPath needs this
import mx.xpath.*;

class ResultsData {
	//EventDispatcher needs these
	var addEventListener:Function;
	var removeEventListener:Function;
	var dispatchEvent:Function;
	var dispatchQueue:Function;
	
	private var gateway:XMLGateway;
	
	private var urlLookup:XML;//<methodName, path>
	private var allQuizIDs:Array;//<id, name>
	private var allUserIDs:Array;//<id, name>
	
	private var userInfo:Object;//{quizList:[id,id]}
	private var quizInfo:Object;//{userList: [id,id]}
	private var quizResults:Object;//{userList: [id,id]}
	private var quizes:Object;//{userList: [id,id]}
	
	private var uniqueID:Number=0;
	
	function ResultsData(){
		EventDispatcher.initialize(this);
		
		gateway=new XMLGateway("POST");
		gateway.addEventListener("onResult", this);
		//can't init until main says to (so its listeners are set up)
		
		//init holders
		userInfo=new Object();
		quizInfo=new Object();
		quizResults=new Object();
		quizes=new Object();
	}
	
	function init(pathToXML:String){
		goGetURLs(pathToXML);
	}
	
	
	
	function onResult(evt){
		//trace(evt.tracker.method)
		switch(evt.tracker.method){
			case "getURLs":
				//save them
				urlLookup=new XML();
				urlLookup.ignoreWhite=true;
				urlLookup=evt.value.firstChild;
				
				//call next method automatically
				goGetAllQuizIDs();
				break;
				
			case "getAllQuizIDs":
				//save off results
				allQuizIDs=XPathAPI.selectNodeList(evt.value.firstChild, "/results/quiz" );
				
				//call next method automatically:
				goGetAllUserIDs();
				break;
			
			case "getAllUserIDs":
				//save off results
				allUserIDs=XPathAPI.selectNodeList(evt.value.firstChild, "/results/user" );
				
				//dispatch that we're ready
				dispatchEvent({type:"onReady", 
				   	 	  		target:this});
				break;
			
			case "getUserQuizIDs":
				//save off results
				var str=evt.value.firstChild.firstChild.nodeValue;
				
				userInfo[evt.tracker.userID]=str.split(",");
				if( str == undefined ){
					//userInfo[evt.tracker.userID] = new Array();
				}
				
				//call the original method (this time it'll go through)
				getUserQuizIDs(evt.tracker.userID,evt.tracker.requestID);
				break;
				
			case "getQuizPercentages":
				//save off results
				var details=XPathAPI.selectNodeList(evt.value.firstChild, "/results/detail" );
				var total=details.length;
				var thisArray=new Array();
				for(var i=0;i<total;i++){
					thisArray.push({id:details[i].attributes.userID, 
						            percentage:Number(details[i].attributes.percentage)})
				}
				quizInfo[evt.tracker.quizID]=thisArray;
				
				//call the original method (this time it'll go through)
				getQuizPercentages(evt.tracker.quizID,evt.tracker.requestID);
				break;
				
			case "getQuizResults":
				//clean it up first
				var questions=XPathAPI.selectNodeList(evt.value.firstChild,"results/questionDetails");
				var total=questions.length;
				var thisArray=new Array();
				for(var i=0;i<total;i++){
					var theseDetails=new Object();
					//stuff the question ID:
					theseDetails.questionID=questions[i].attributes.questionID;

					//stuff an array of correct answers
					theseDetails.correctAnswers=questions[i].attributes.correctAnswers.split(",");
					
					//prepare an array of user results
					theseDetails.userResults=new Array();
					var resultsForThisQ=XPathAPI.selectNodeList(questions[i],"questionDetails/userAnswer");
					var totalUsers=resultsForThisQ.length;
					for(var j=0;j<totalUsers;j++){
						theseDetails.userResults.push({id:resultsForThisQ[j].attributes.userID,
													  theirAnswers:resultsForThisQ[j].attributes.theirAnswers.split(",")});
					}
					
					thisArray.push(theseDetails)
				}
		
				
				quizResults[evt.tracker.quizID]=thisArray;
				
				//call the original method (this time it'll go through)
				getQuizResults(evt.tracker.quizID, evt.tracker.requestID);
				break;
				
			case "getQuiz":
				//store a Quiz obj into the quizes cache
				quizes[evt.tracker.quizID]=new Quiz( evt.value.firstChild);
				    
				//call the original method (this time it'll go through)
				getQuiz(evt.tracker.quizID, evt.tracker.requestID);
				break;
		}
	}
	
	
	//===========================
	//calls directed to gateway:
	//===========================
	function goGetURLs(pathToXML:String){
		gateway.trigger(true, {method:"getURLs"}, pathToXML, new LoadVars());
	}
	function goGetAllQuizIDs(){
		gateway.trigger(true, {method:"getAllQuizIDs"}, getPath("getAllQuizIDs"), new LoadVars());
	}
	function goGetAllUserIDs(){
		gateway.trigger(true, {method:"getAllUserIDs"}, getPath("getAllUserIDs"), new LoadVars());
	}
	function goGetUserQuizIDs(userID:String, requestID:Object){
		var objToSend=new LoadVars();
		objToSend.userID=userID;
		gateway.trigger(true,
						{method:"getUserQuizIDs",
						 userID:userID, 
						 requestID:requestID}, 
						getPath("getUserQuizIDs"), 
						objToSend);
	}
	
	function goGetQuizPercentages(quizID:String, requestID:Object){
		var objToSend=new LoadVars();
		objToSend.quizID=quizID;
		gateway.trigger(true,
						{method:"getQuizPercentages",
						 quizID:quizID, 
						 requestID:requestID}, 
						getPath("getQuizPercentages"), 
						objToSend);
		
	}
	function goGetQuizResults(quizID:String, requestID:Object){
		var objToSend=new LoadVars();
		objToSend.quizID=quizID;
		gateway.trigger(true,
						{method:"getQuizResults",
						 quizID:quizID, 
						 requestID:requestID}, 
						getPath("getQuizResults"), 
						objToSend);
	}

	function goGetQuiz(quizID:String, requestID:Object){
		var objToSend=new LoadVars();
		objToSend.quizID=quizID;
		gateway.trigger(true,
						{method:"getQuiz",
						 quizID:quizID, 
						 requestID:requestID}, 
						getPath("getQuiz"), 
						objToSend);
	}
	

	//===========================
	//PUBLIC METHODS
	//===========================	
	
	//==========
	//requiring callback (asynchro):
	public function getUserQuizIDs(userID:String, requestID:Object){
		if(userInfo[userID]==undefined){
			//trace("indirect");
			goGetUserQuizIDs(userID, requestID);
		}else{
			//trace("direct");
			dispatchEvent({type:"gotResults",
						   tracker:{method:"getUserQuizIDs",userID:userID,requestID:requestID},
				   	 	   value:userInfo[userID]});
		}
	}
	public function getQuizPercentages(quizID:String, requestID:Object){
		if(quizInfo[quizID]==undefined){
			//trace("indirect");
			goGetQuizPercentages(quizID, requestID);
		}else{
			//trace("direct");
			dispatchEvent({type:"gotResults",
						   tracker:{method:"getQuizPercentages",quizID:quizID,requestID:requestID},
				   	 	   value:quizInfo[quizID]});
		}
	}
	
	public function getQuizResults(quizID:String, requestID:Object){
		if(quizResults[quizID]==undefined){
			//trace("indirect");
			goGetQuizResults(quizID, requestID);
		}else{
			//trace("direct");
			dispatchEvent({type:"gotResults",
						   tracker:{method:"getQuizResults",quizID:quizID,requestID:requestID},
				   	 	   value:quizResults[quizID]});
		}
	}
	
	public function getQuiz(quizID:String, requestID:Object){
		if(quizes[quizID]==undefined){
			//trace("indirect");
			goGetQuiz(quizID, requestID);
		}else{
			//trace("direct");
			dispatchEvent({type:"gotResults",
						   tracker:{method:"getQuiz",quizID:quizID,requestID:requestID},
				   	 	   value:quizes[quizID]});
		}
	}
	
	
	//==========
	//synchronous	
	public function getAllQuizes():Array{ // of {id, name}
		var total=allQuizIDs.length;
		var retArray=new Array();
		for(var i=0;i<total;i++){
			retArray.push({name:allQuizIDs[i].attributes.name, 
				           id:allQuizIDs[i].attributes.quizID})
		}
		return retArray;
	}
	public function getAllUsers():Array{ // of {id, name}
		var total=allUserIDs.length;
		var retArray=new Array();
		for(var i=0;i<total;i++){
			retArray.push({name:allUserIDs[i].attributes.name, 
				           id:allUserIDs[i].attributes.userID})
		}
		return retArray;
	}
	
	
	
	//===========================
	//UTILITIES
	//===========================
	function getPath(methodName){
		var str="/allPaths/url[@methodName=\""+methodName+"\"]";
		return XPathAPI.selectSingleNode(urlLookup, str ).attributes.path;
	}

	public function getQuizByID(id:String):String{
		var total=allQuizIDs.length;
		for(var i=0;i<total;i++){
			if(allQuizIDs[i].attributes.quizID==id){
				return allQuizIDs[i].attributes.name;
			}
		}
		return "["+id+"]";
	}
	public function getUserByID(id:String):String{
		var total=allUserIDs.length;
		for(var i=0;i<total;i++){
			if(allUserIDs[i].attributes.userID==id){
				return allUserIDs[i].attributes.name;
			}
		}
		return "["+id+"]";
	}
	public function getResultsByQuestionID(questionID:String, quizID:String):Object{
		var wholeQuiz:Array=quizResults[quizID];
		var total:Number=wholeQuiz.length;
		var retObj:Object=new Object();
		for(var i=0;i<total;i++){
			var thisQuestion=wholeQuiz[i];
			if(thisQuestion.questionID == questionID){
				retObj.correctAnswers=thisQuestion.correctAnswers;
				retObj.userResults=thisQuestion.userResults;
			}
		}
		return retObj;
	}
	

	
	
}